import sys
import scipy.io.wavfile
import os
import curses
import fnmatch
import numpy as np
os.environ["CUDA_VISIBLE_DEVICES"]="-1" 
import string
import time
import tensorflow as tf
from keras.preprocessing.sequence import pad_sequences
from keras.layers import Dense, LSTM, Reshape, BatchNormalization, Input,  Lambda, Bidirectional, Conv2D, MaxPool2D, Flatten
from keras.models import Model
from keras.activations import relu, sigmoid, softmax
import keras.backend as K
from keras.utils import to_categorical
from keras.callbacks import ModelCheckpoint
import sounddevice as sd
import queue as q
from keras.models import model_from_json
import gc
from keras.backend import eval

FRAME = 65536
DURATION = 8192
FRAMERATE = 8000
TIMESTEPS = 128
CHUNK = FRAMERATE*DURATION // (TIMESTEPS * 1000) 
# The size of a sample in chunks
SEQ_LENGTH = ((FRAMERATE * DURATION) // (CHUNK *100)) * CHUNK
SIGLENGTH = (FRAMERATE * DURATION) // 1000
MAXTEST = 100
timesteps = FRAME // CHUNK
maxline = 80

# The character set in canonical order
char_list = [' ','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z','0','1',
             '2','3','4','5','6','7','8','9','?','/', ':', '>', '<', '=', '+', '-', '!', ',', '\0']
NUM_CLASSES = len(char_list)


def encode_to_labels(txt):
    # encoding each output word into digits
    dig_lst = []
    for index, char in enumerate(txt):
        try:
            dig_lst.append(char_list.index(char))
        except:
            print(char)
        
    return dig_lst

def makejoin(s1, s2):
    #print('s2=',s2,'/ls2=',len(s2))
    if (len(s2) < 3):
         return(s1+' ')
    lhook=len(s2)
    max = 0
    realfirst = 0
    tmphook = 0
    for j in range(1,lhook):
      #print('jhook=',j)
      hook=len(s1)-1
      s1last = s1[hook-j:]
      ls1last=len(s1last)
      #print('s1last=',s1last, '/lens1last=',ls1last)
      same = 0
      first = 0
      for i in range(j):
        #print('i=',i)
        #print('paar=',s2[i],'/',s1last[i],'/i=',i)
        if (s2[i] == s1last[i]): 
          same += 1
          if ( first == 0):
            first = i
      if (same > max):
          max = same
          realfirst= first
          tmphook = j
      #print('j was=',j,'same=', same, 'first=', first, 'max=', max, 'realfirst=', realfirst, 'tmphook=', tmphook)
    #print('max=', max, 'realfirst=', realfirst, 'hook=', hook) 
    #print('s1deel=',s1[:realfirst + hook - tmphook], '/s2deel=', s2[realfirst:],'/')
    if (realfirst == 0):
          return(s1+s2)
    else:
      return (s1[:hook-tmphook + realfirst] + s2[realfirst:])



if K.image_data_format() == 'channels_first':
    inputs = Input(shape=(1, SIGLENGTH))
else:
    inputs = Input(shape=(SIGLENGTH,1))

# input with shape  128x  512

# load json and create model
json_file = open('neuralnetcw.json', 'r')
loaded_model_json = json_file.read()
json_file.close()
act_model = model_from_json(loaded_model_json)
#act_model.summary()

#load the model weights
act_model.load_weights('best_modelfast.hdf5')

#data acquisition and evaluation 
ibuf = sd.rec(channels=1, samplerate=8000, frames=65536, dtype='int16')
sd.wait()
inbuf1 = ibuf
ibuf = sd.rec(channels=1, samplerate=8000, frames=65536, dtype='int16')
test_sigs= []
data = np.asarray(inbuf1, dtype=np.float32)
data = data.reshape(1, timesteps, CHUNK)
data =np.expand_dims(data, axis=3)
test_sigs.append(data)
prediction = act_model.predict(test_sigs)
nep =K.ctc_decode(prediction, input_length=np.ones(prediction.shape[0])*prediction.shape[1],
                     greedy=True)
out = eval(nep[0][0])
# see the results
screen = curses.initscr()
screen.keypad(True)
curses.cbreak()
screen.clear()
screen.refresh()


txtout1 =''
for x in out:
   for p in x:  
      if int(p) != -1:
         txtout1 = txtout1 + char_list[int(p)]    

myq = q.Queue()

def callback(indata, frames, time, status):
    """This is called (from a separate thread) for each audio block."""
    if status:
        print(status, file=sys.stderr)
    myq.put(indata.copy())

# initializations

tmpbuf = np.zeros(FRAME, dtype='int16')
txtout2 = ' '
fulltxt = 'pa0mbo                          '
mywin = curses.newwin(5,120)
screen.refresh()
# now loop
with sd.InputStream(samplerate=8000,  channels=1, blocksize=FRAME//4, dtype='int16', callback=callback):
   while True:
     ibuf = myq.get()
     test_sigs= []
     #shift data to begin of buffer and add new data at second half
     for i in range(0,(FRAME//4)):
          tmpbuf[i] = tmpbuf[(FRAME//4) + i]
          tmpbuf[(FRAME//4) + i] = ibuf[i]
     data = np.asarray(tmpbuf, dtype=np.float32)
     data = data.reshape(1, timesteps, CHUNK)
     data =np.expand_dims(data, axis=3)
     test_sigs.append(data)
     prediction = act_model.predict(test_sigs)
     # use CTC decoder
     nep  = K.ctc_decode(prediction, input_length=np.ones(prediction.shape[0])*prediction.shape[1],
                         greedy=True)
     out = eval(nep[0][0])
     # calc the results
     txtout2 = ''
     for x in out:
       cntx=0
       for p in x:
          cntx += 1
          #print('cntx=', cntx, 'int_p=', int(p))  
          if int(p) != -1:
             txtout2 = txtout2 + char_list[int(p)]
     txtout3 = txtout2[:len(txtout2)-1] 
     fulltxt = makejoin(fulltxt, txtout3)
     lfulltxt = len(fulltxt)
     if lfulltxt > 120:
        fulltxt = fulltxt[lfulltxt -100 :]
        mywin.clear()
     mywin.addstr(1,0, fulltxt)
     mywin.clrtoeol() 
     mywin.refresh()    
     gc.collect()
 
    
   


